unit XAligns;

interface

uses Windows, XClasses, XWindows, XApplets, XForms, XVisuals;

type
   XAligner = class
   private
      procedure GetFormAvailableClientRect( Sender : TObject; var Rect : TRect );
      procedure RealignForm( Sender : TObject );
   protected
   public
      constructor Create;
   end;

var Aligner : XAligner;

procedure UseAligner;

implementation

procedure UseAligner;
begin
   if Aligner = nil then
      Aligner := XAligner.Create;
end;

constructor XAligner.Create;
begin
   inherited;
   GlobalFormsManager.On_GetAvailableClientRect :=
         GetFormAvailableClientRect;
   GlobalFormsManager.On_Realign := RealignForm;
end;

procedure XAligner.GetFormAvailableClientRect( Sender : TObject; var Rect : TRect );
var I : Integer;
    C : XClass;
    E : XControl;
begin
   Rect := XRect( 0, 0, 0, 0 );
   if not (Sender is XVisual) then
      Exit;
   with (Sender as XVisual) do
   for I := 0 to ChildsCount - 1 do
   begin
      C := Childs[ I ];
      if C is XControl then
      begin
         E := C as XControl;
         if E.Align <> alNone then
         begin
            case E.Align of
            alClient: begin
                         Rect := XRect( 0, 0, 0, 0 );
                         Exit;
                      end;
            alLeft:   begin
                         if Rect.Left < E.Left + E.Width then
                            Rect.Left := E.Left + E.Width;
                      end;
            alTop:    begin
                         if Rect.Top < E.Top + E.Height then
                            Rect.Top := E.Top + E.Height;
                      end;
            alRight:  begin
                         if Rect.Right > E.Left then
                            Rect.Right := E.Left;
                      end;
            alBottom: begin
                         if Rect.Bottom > E.Top then
                            Rect.Bottom := E.Top;
                      end;
            end;
         end;
         if (Rect.Left >= Rect.Right) or
            (Rect.Top >= Rect.Bottom) then
         begin
            Rect := XRect( 0, 0, 0, 0 );
            Exit;
         end;
      end;
   end;
end;

procedure XAligner.RealignForm( Sender : TObject );
var V : XVisual;
    Ch : XClass;
    E : XControl;
    I, W, H, X1, X2, Y1, Y2 : Integer;
    R : TRect;
    ChangePos, ChangeSiz : Boolean;
begin
   V := Sender as XVisual;
   R := V.ClientRect;
   W := R.Right;
   H := R.Bottom;
   X1 := 0;
   X2 := W;
   Y1 := 0;
   Y2 := H;
   for I := 0 to V.ChildsCount - 1 do
   begin
      Ch := V.Childs[ I ];
      if Ch is XControl then
      begin
         E := Ch as XControl;
         if not E.Visible then
            continue;
         R := E.BoundsRect;
         if E.Align in [ alTop, alBottom ] then
         begin
            R.Left := 0;
            R.Right := W;
            if E.Align = alBottom then
            begin
               R.Top := Y2 - E.Height;
               R.Bottom := Y2;
               if Y2 > R.Top then
                  Y2 := R.Top;
            end
               else
            begin
               R.Top := Y1;
               R.Bottom := Y1 + E.Height;
               if Y1 < R.Bottom then
                  Y1 := R.Bottom;
            end;
         end;
         ChangePos := (R.Top <> E.Top) or (R.Left <> E.Left);
         ChangeSiz := (R.Right - R.Left <> E.Width) or (R.Bottom - R.Top <> E.Height);
         if ChangePos or ChangeSiz then
         begin
            E.BoundsRect := R;
            if ChangeSiz then
               RealignForm( E );
         end;
      end;
   end;
   for I := 0 to V.ChildsCount - 1 do
   begin
      Ch := V.Childs[ I ];
      if Ch is XControl then
      begin
         E := Ch as XControl;
         if not E.Visible then
            continue;
         R := E.BoundsRect;
         if E.Align in [ alLeft, alRight ] then
         begin
            R.Top := Y1;
            R.Bottom := Y2;
            if E.Align = alRight then
            begin
               R.Left := X2 - E.Width;
               R.Right := X2;
               if X2 > R.Left then
                  X2 := R.Left;
            end
               else
            begin
               R.Left := X1;
               R.Right := X1 + E.Width;
               if X1 < R.Right then
                  X1 := R.Right;
            end;
         end;
         ChangePos := (R.Top <> E.Top) or (R.Left <> E.Left);
         ChangeSiz := (R.Right - R.Left <> E.Width) or (R.Bottom - R.Top <> E.Height);
         if ChangePos or ChangeSiz then
         begin
            E.BoundsRect := R;
            if ChangeSiz then
               RealignForm( E );
         end;
      end;
   end;
   for I := 0 to V.ChildsCount - 1 do
   begin
      Ch := V.Childs[ I ];
      if Ch is XControl then
      begin
         E := Ch as XControl;
         if not E.Visible then
            continue;
         R := E.BoundsRect;
         if E.Align = alClient then
         begin
            R.Top := Y1;
            R.Bottom := Y2;
            R.Left := X1;
            R.Right := X2;
            if (R.Top >= R.Bottom) or (R.Left >= R.Right) then
            begin
               R.Bottom := R.Top;
               R.Right := R.Left;
            end;
            X2 := X1;
            Y2 := Y1;
         end;
         ChangePos := (R.Top <> E.Top) or (R.Left <> E.Left);
         ChangeSiz := (R.Right - R.Left <> E.Width) or (R.Bottom - R.Top <> E.Height);
         if ChangePos or ChangeSiz then
         begin
            E.BoundsRect := R;
            if ChangeSiz then
               RealignForm( E );
         end;
      end;
   end;
end;

initialization

finalization

   Aligner.Free;

end.
