unit XControls;

interface

uses Windows, XClasses, XVisuals, XWindows, XForms, XCanvases, XGraphics;

type
   XCustomControl = class( XControl )
   private
      fOnChangeAppearance : XOnEvent;
   protected
      procedure DoPaint( DC : HDC; Rect : TRect ); override;
      procedure ChangeAppearance;
   public
      procedure PaintTo( C : XCanvas; Rect : TRect ); dynamic;
      property OnChangeAppearance : XOnEvent read fOnChangeAppearance write fOnChangeAppearance;
      function CanFocus : Boolean; override;
   end;

   XLabel = class( XCustomControl )
   private
      fAutoSize : Boolean;
      fCaption : String;
      fFont : XFont;
      fAlignment : XTextAlignment;
      procedure SetAutoSize( Value : Boolean );
      procedure SetCaption( Value : String );
      procedure SetAlignment( Value : XTextAlignment );
      procedure FontChanged( Sender : TObject );
   protected
      procedure Resize; dynamic;
   public
      constructor Create( AParent : XClass ); override;
      destructor Destroy; override;
      property AutoSize : Boolean read fAutoSize write SetAutoSize;
      property Caption : String read fCaption write SetCaption;
      property Font : XFont read fFont;
      property Color;
      property Alignment : XTextAlignment read fAlignment write SetAlignment;
      procedure PaintTo( C : XCanvas; Rect : TRect ); override;
   end;

// I recommend to use function below instead of direct call of XLabel
// constructor, because it allow using add-ons to replace all labels
// (including hidden internal ones used in other controls) with
// another XCustomLabel descenders (may be improved). I especially
// recommend to use this function when designing such complex
// controls, which use XLabel as parts of itself similar XGroup,
// XButton and so on.

function NewLabel( AParent : XClass; Caption : String; X, Y : Integer ) : XLabel;

type
   XBevelStyle = (bvNone, bvRaised, bvLowered);
   XBevelEdge = (beLeft, beTop, beRight, beBottom);
   XBevelEdges = Set of XBevelEdge;

   XCustomBevel = class( XCustomControl )
   private
      fBevelEdges : XBevelEdges;
      fBevelInner : XBevelStyle;
      fBevelOuter : XBevelStyle;
      fBevelWidth : Integer;
      fBorderWidth : Integer;
      procedure SetBevelEdges( Value : XBevelEdges );
      procedure SetBevelInner( Value : XBevelStyle );
      procedure SetBevelOuter( Value : XBevelStyle );
      procedure SetBevelWidth( Value : Integer );
      procedure SetBorderWidth( Value : Integer );
   protected
      property BevelEdges : XBevelEdges read fBevelEdges write SetBevelEdges;
      property BevelInner : XBevelStyle read fBevelInner write SetBevelInner;
      property BevelOuter : XBevelStyle read fBevelOuter write SetBevelOuter;
      property BevelWidth : Integer read fBevelWidth write SetBevelWidth;
      property BorderWidth: Integer read fBorderWidth write SetBorderWidth;
      property Color;
   public
      constructor Create( AParent : XClass ); override;
      function ClientRect : TRect; override;
      procedure PaintTo( C : XCanvas; Rect : TRect ); override;
      function CanFocus : Boolean; override;
   end;

   XBevel = class( XCustomBevel )
   private
   protected
   public
      property BevelEdges;
      property BevelInner;
      property BevelOuter;
      property BevelWidth;
      property BorderWidth;
      property Color;
   end;

   XPanelLayout = ( plCenter, plTopLeft, plTop, plTopRight, plLeft, plRight,
                    plBottomLeft, plBottom, plBottomRight );
   XPanel = class( XBevel )
   private
      fTitle : XLabel;
      fLayout : XPanelLayout;
      function GetCaption : String;
      procedure SetCaption( Value : String );
      procedure SetLayout( Value : XPanelLayout );
      procedure SetColor( Value : XColor ); override;
      function GetFont : XFont;
      procedure AppearanceChanged( Sender : TObject );
   protected
      procedure SetOwnerForm( Value : XForm ); override;
   public
      constructor Create( AParent : XClass ); override;
      destructor Destroy; override;
      property Caption : String read GetCaption write SetCaption;
      property Layout : XPanelLayout read fLayout write SetLayout;
      procedure PaintTo( C : XCanvas; Rect : TRect ); override;
      property BevelInner;
      property BevelOuter;
      property BevelWidth;
      property BorderWidth;
      property Color;
      property Font : XFont read GetFont;
   end;

   XGroupLayout = ( glTopLeft, glTopCenter, glTopRight,
                    glBottomLeft, glBottomCenter, glBottomRight,
                    glLeftBottom, glLeftCenter, glLeftTop,
                    glRightTop, glRightCenter, glRightBottom );

   XGroup = class( XCustomBevel )
   private
      fTitle : XLabel;
      fFrame : XBevel;
      fLayout : XGroupLayout;
      function GetBevel : XBevelStyle;
      procedure SetBevel( Value : XBevelStyle );
      procedure SetLayout( Value : XGroupLayout );
      function GetCaption : String;
      procedure SetCaption( Value : String );
      function GetFont : XFont;
      procedure SetColor( Value : XColor ); override;
      procedure AppearanceChanged( Sender : TObject );
   protected
      procedure SetOwnerForm( Value : XForm ); override;
   public
      constructor Create( AParent : XClass ); override;
      destructor Destroy; override;
      property Bevel : XBevelStyle read GetBevel write SetBevel;
      property Layout : XGroupLayout read fLayout write SetLayout;
      property Caption : String read GetCaption write SetCaption;
      property Font : XFont read GetFont;
      procedure PaintTo( C : XCanvas; Rect : TRect ); override;
      function ClientRect : TRect; override;
      property Color;
   end;

   XOnPaint = procedure( Sender : TObject; Canvas : XCanvas; Rect : TRect )
              of object;

   XPaint = class( XCustomControl )
   private
      fOnPaint : XOnPaint;
   protected
   public
      procedure PaintTo( Canvas : XCanvas; Rect : TRect ); override;
      property OnPaint : XOnPaint read fOnPaint write fOnPaint;
      property Color;
   end;

   XCustomCheckBox = class( XBevel )
   private
      fChecked : Boolean;
      fCheckColor : XColor;
      fGrayDisabled : Boolean;
      fOnChanged : XOnEvent;
      fPaintColor : XColor;
      procedure SetCheckColor( Value : XColor );
      procedure SetGrayDisabled( Value : Boolean );
      procedure Set_Color( Value : XColor );
      procedure SetEnabled( Value : Boolean ); override;
   protected
      fGroup : XControl;
      procedure SetChecked( Value : Boolean ); virtual;
      procedure SetGroup( Value : XControl );
      property Group : XControl read fGroup write SetGroup;
      function RadioAlternate( UnCheck : Boolean ) : Boolean;
   public
      constructor Create( AParent : XClass ); override;
      property Checked : Boolean read fChecked write SetChecked;
      property BoxColor : XColor read fPaintColor write Set_Color;
      property CheckColor : XColor read fCheckColor write SetCheckColor;
      property GrayDisabled : Boolean read fGrayDisabled write SetGrayDisabled;
      property OnChanged : XOnEvent read fOnChanged write fOnChanged;
      property Color;
      function CanFocus : Boolean; override;
      procedure Click; dynamic;
      procedure DoMouseDown( Button : XMouseButton; Shift : XShiftState; X, Y : Integer ); override;
      procedure DoKeyDown( var Key : Word; Shift : XShiftState ); override;
   end;

   XCheckBox = class( XCustomCheckBox )
   private
   protected
   public
      procedure PaintTo( C : XCanvas; Rect : TRect ); override;
   end;

   XRadioBox = class( XCustomCheckBox )
   private
   protected
      procedure SetChecked( Value : Boolean ); override;
   public
      constructor Create( AParent : XClass ); override;
      procedure PaintTo( C : XCanvas; Rect : TRect ); override;
      property Group;
      procedure Click; override;
   end;

   XBoxLayout = ( blRight, blLeft, blBottom, blTop );

   XCustomCheck = class( XCustomCheckBox )
   private
      fBox : XCustomCheckBox;
      fTitle : XLabel;
      fLayout : XBoxLayout;
      fAutoSize : Boolean;
      function GetCaption : String;
      procedure SetCaption( Value : String );
      procedure SetAutoSize( Value : Boolean );
      procedure SetLayout( Value : XBoxLayout );
      function GetFont : XFont;
      procedure Resize( Sender : TObject );
   protected
      procedure CreateBox; dynamic;
      procedure SetChecked( Value : Boolean ); override;
      procedure SetOwnerForm( Value : XForm ); override;
   public
      constructor Create( AParent : XClass ); override;
      destructor Destroy; override;
      property AutoSize : Boolean read fAutoSize write SetAutoSize;
      property Caption : String read GetCaption write SetCaption;
      property Layout : XBoxLayout read fLayout write SetLayout;
      property Font : XFont read GetFont;
      procedure PaintTo( C : XCanvas; Rect : TRect ); override;
   end;

   XCheck = class( XCustomCheck )
   private
      procedure CreateBox; override;
   public
   end;

   XRadio = class( XCustomCheck )
   private
      procedure CreateBox; override;
      procedure SetChecked( Value : Boolean ); override;
   public
      constructor Create( AParent : XClass ); override;
      property Group;
      procedure Click; override;
   end;

   XCustomButton = class( XCustomBevel )
   private
    FFlat : Boolean;
    FDown: Boolean;
    FFixed: Boolean;
    FSpeedButton: Boolean;
    fOnClick : XOnEvent;
    fKeyUsed : Word;
    FRepeats: Boolean;
    procedure SetFlat(const Value: Boolean);
    procedure SetDown(const Value: Boolean);
    procedure SetFixed(const Value: Boolean);
    procedure SetSpeedButton(const Value: Boolean);
   protected
   public
    constructor Create( AParent : XClass ); override;
    property Flat : Boolean read FFlat write SetFlat;
    property Down : Boolean read FDown write SetDown;
    property Fixed : Boolean read FFixed write SetFixed;
    function CanFocus : Boolean; override;
    property SpeedButton : Boolean read FSpeedButton write SetSpeedButton;
    property Repeats : Boolean read FRepeats write FRepeats;
    property Color;
    property OnClick : XOnEvent read fOnClick write fOnClick;
    procedure DoMouseDown( Button : XMouseButton; Shift : XShiftState;
                           X, Y : Integer ); override;
    procedure DoMouseUp( Button : XMouseButton; Shift : XShiftState;
                           X, Y : Integer ); override;
    procedure DoKeyDown( var Key : Word; Shift : XShiftState ); override;
    procedure DoKeyUp( var Key : Word; Shift : XShiftState ); override;
   end;

   XButton = class(XCustomButton)
   private
    fTitle : XLabel;
    FAutoSize: Boolean;
    function GetCaption: String;
    procedure SetAutoSize(const Value: Boolean);
    procedure SetCaption(const Value: String);
    procedure Resize( Sender : TObject );
    function GetFont: XFont;
   protected
    procedure SetColor( Value : XColor ); override;
   public
    constructor Create( AParent : XClass ); override;
    destructor Destroy; override;
    property Caption : String read GetCaption write SetCaption;
    property Font : XFont read GetFont;
    property AutoSize : Boolean read FAutoSize write SetAutoSize;
    procedure PaintTo( C : XCanvas; Rect : TRect ); override;
   end;

   XOnCreateLabel = procedure( AParent : XClass; const Caption : String; X, Y : Integer;
                               var ALabel : XLabel ) of object;
   XOnPaintLabel = procedure( Sender : TObject; Canvas : XCanvas; Rect : TRect;
                              var Handled : Boolean ) of object;
   XControlsManager = class
   private
    FOnCreateLabel: XOnCreateLabel;
    FOnPaintLabel: XOnPaintLabel;
   protected
   public
     property OnCreateLabel : XOnCreateLabel read FOnCreateLabel write FOnCreateLabel;
     property On_PaintLabel : XOnPaintLabel read FOnPaintLabel write FOnPaintLabel;
   end;

var GlobalControlsManager : XControlsManager;

implementation

procedure XCustomControl.DoPaint( DC : HDC; Rect : TRect );
var C : XCanvas;
    F : XForm;
begin
   PaintChilds( DC, Rect );
   F := OwnerForm;
   if F = nil then Exit;
   C := WindowCanvas( F );
   PaintTo( C, Rect );
end;

procedure XCustomControl.PaintTo( C : XCanvas; Rect : TRect );
var F : XForm;
begin
   F := ParentForm;
   if (F <> nil) and (C = WindowCanvas( F )) then
      ExcludeUpdRect( C.Handle, Rect )
   else
      ExcludeClipRect( C.Handle, Rect.Left, Rect.Top, Rect.Right, Rect.Bottom );
end;

function XCustomControl.CanFocus : Boolean;
begin
   Result := False;
end;

procedure XCustomControl.ChangeAppearance;
begin
   if Assigned( fOnChangeAppearance ) then
      fOnChangeAppearance( Self );
end;

///////////////////////////////////////////////////////////////////
//                                                               //
//    XX    XX   XX           XX      XXXXXXX   XXXXXX  XX       //
//     XX XX     XX          XXXX     XX    XX  XX      XX       //
//      XX       XX         XX  XX    XXXXXXX   XXXX    XX       //
//    XX XX      XX        XXXXXXXX   XX    XX  XX      XX       //
//  XX    XX     XXXXXXX  XX      XX  XXXXXXX   XXXXXX  XXXXXXX  //
//                                                               //
///////////////////////////////////////////////////////////////////

constructor XLabel.Create( AParent : XClass );
begin
   inherited;
   fAutoSize := True;
   Height := 16;
   fFont := XFont.Create;
   fFont.OnChange := FontChanged;
end;

destructor XLabel.Destroy;
begin
   fFont.Free;
   inherited;
end;

procedure XLabel.SetCaption( Value : String );
begin
   if Value = fCaption then Exit;
   fCaption := Value;
   if AutoSize then
      Resize;
   Invalidate;
end;

procedure XLabel.SetAutoSize( Value : Boolean );
begin
   if Value = fAutoSize then Exit;
   fAutoSize := Value;
   if Value then
      Resize;
end;

procedure XLabel.FontChanged( Sender : TObject );
begin
   if fAutoSize then
   begin
     Resize;
     if Assigned( fOnChangeAppearance ) then
        fOnChangeAppearance( Self );
   end;
   Invalidate;
end;

procedure XLabel.Resize;
var R : TRect;
    C : XCanvas;
    F : XForm;
    OldFont : XFont;
    Sz : TSize;
    Pt : TPoint;
begin
   F := OwnerForm;
   if F = nil then Exit;
   R := BoundsRect;
   C := WindowCanvas( F as XWindow );
   OldFont := XFont.Create; OldFont.Assign( C.Font );
   C.Font.Assign( Font );
   if Font.Orientation = 0 then
   begin
      R.Right := R.Left + C.TextWidth( Caption );
      R.Bottom := R.Top + C.TextHeight( Caption + 'Ap' );
   end
      else
   begin
      C.TextArea( Caption, Sz, Pt );
      R.Right := R.Left + Sz.cx;
      R.Bottom := R.Top + Sz.cy;
   end;
   C.Font.Assign( OldFont );
   OldFont.Free;
   BoundsRect := R;
end;

procedure XLabel.PaintTo( C : XCanvas; Rect : TRect );
var DstRgn, DrwRgn : HRgn;
    DrwRect : TRect;
    Origin : TPoint;
    E : TSize;
    P : TPoint;
    OldFont : XFont;
    OldBrush : XBrush;
    Handled : Boolean;
begin
   if Assigned( GlobalControlsManager.On_PaintLabel ) then
   begin
      Handled := False;
      GlobalControlsManager.On_PaintLabel( Self, C, Rect, Handled );
      if Handled then
      begin
        inherited;
        Exit;
      end;
   end;
   OldFont := XFont.Create;
   OldFont.Assign( C.Font );
   C.Font.Assign( Font );
   DstRgn := CreateRectRgn( Rect.Left, Rect.Top, Rect.Right, Rect.Bottom );
   C.TextArea( Caption, E, P );
   if E.cx > Rect.Right - Rect.Left then
      E.cx := Rect.Right - Rect.Left;
   if E.cy > Rect.Bottom - Rect.Top then
      E.cy := Rect.Bottom - Rect.Top;
   Origin := Rect.TopLeft;
   case fAlignment of
   taRight : Origin.x := Rect.Right - E.cx;
   taCenter: Origin.x := Rect.Left + (Rect.Right - Rect.Left - E.cx) div 2;
   end;
   DrwRect := XRect( Origin.x, Origin.y, Origin.x + E.cx, Origin.y + E.cy );
   DrwRgn := CreateRectRgn( DrwRect.Left, DrwRect.Top, DrwRect.Right, DrwRect.Bottom );
   OldBrush := XBrush.Create; OldBrush.Assign( C.Brush );
   C.Brush.Color := Color;
   C.Brush.Style := bsSolid;
   C.Brush.Bitmap := 0;
   C.TextRect( DrwRect, Rect.Left + P.x, Rect.Top + P.y, Caption );
   if CombineRgn( DstRgn, DstRgn, DrwRgn, RGN_DIFF ) <> NULLREGION then
   begin
      GetClipRgn( C.Handle, DrwRgn );
      ExtSelectClipRgn( C.Handle, DstRgn, RGN_AND );
      C.FillRect( Rect );
      SelectClipRgn( C.Handle, DrwRgn );
   end;
   C.Brush.Assign( OldBrush ); OldBrush.Free;
   C.Font.Assign( OldFont ); OldFont.Free;
   DeleteObject( DstRgn );
   DeleteObject( DrwRgn );
   inherited;
end;

procedure XLabel.SetAlignment( Value : XTextAlignment );
begin
   if fAlignment = Value then Exit;
   fAlignment := Value;
   Invalidate;
end;

//////////////////////////////////////////////////////////////////
//                                                              //
//     XX    XX  XXXXXXX   XXXXXX  XX      XX  XXXXXX  XX       //
//      XX XX    XX    XX  XX       XX    XX   XX      XX       //
//       XX      XXXXXXX   XXXX      XX  XX    XXXX    XX       //
//     XX XX     XX    XX  XX         XXXX     XX      XX       //
//   XX    XX    XXXXXXX   XXXXXX      XX      XXXXXX  XXXXXXX  //
//                                                              //
//////////////////////////////////////////////////////////////////

constructor XCustomBevel.Create( AParent : XClass );
begin
   inherited;
   BoundsRect := XRect( 0, 0, 40, 40 );
   fBevelEdges := [ beLeft, beTop, beRight, beBottom ];
   fBevelInner := bvLowered;
   fBevelOuter := bvRaised;
   fBevelWidth := 1;
   fBorderWidth := 1;
end;

function XCustomBevel.CanFocus : Boolean;
begin
   Result := ChildsCount > 0;
end;

function XCustomBevel.ClientRect : TRect;
var D : Integer;
begin
   D := 0;
   if fBevelOuter <> bvNone then
   D := fBevelWidth + fBorderWidth;
   if fBevelInner <> bvNone then
      D := D + fBevelWidth;
   Result := XRect( 0, 0, Width, Height );
   if beLeft in fBevelEdges then
      Inc( Result.Left, D );
   if beTop in fBevelEdges then
      Inc( Result.Top, D );
   if beRight in fBevelEdges then
      Dec( Result.Right, D );
   if beBottom in fBevelEdges then
      Dec( Result.Bottom, D );
end;

procedure XCustomBevel.SetBevelEdges( Value : XBevelEdges );
begin
   if fBevelEdges = Value then Exit;
   fBevelEdges := Value;
   Invalidate;
end;

procedure XCustomBevel.SetBevelInner( Value : XBevelStyle );
begin
   if fBevelInner = Value then Exit;
   fBevelInner := Value;
   Invalidate;
end;

procedure XCustomBevel.SetBevelOuter( Value : XBevelStyle );
begin
   if fBevelOuter = Value then Exit;
   fBevelOuter := Value;
   Invalidate;
end;

procedure XCustomBevel.SetBevelWidth( Value : Integer );
begin
   if fBevelWidth = Value then Exit;
   fBevelWidth := Value;
   Invalidate;
end;

procedure XCustomBevel.SetBorderWidth( Value : Integer );
begin
   if fBorderWidth = Value then Exit;
   fBorderWidth := Value;
   Invalidate;
end;

procedure XCustomBevel.PaintTo( C : XCanvas; Rect : TRect );
var OldPen : XPen;
    Light, Shadow, C1, C2 : XColor;
    R : TRect;
    OldBrush : XBrush;
    procedure DrawBevels( C1, C2 : XColor; W : Integer );
    var I : Integer;
        R1 : TRect;
    begin
       for I := 1 to W do
       begin
          R1 := R;
          if beLeft in fBevelEdges then
          begin
             C.Pen.Color := C1;
             C.MoveTo( R.Left, R.Top );
             C.LineTo( R.Left, R.Bottom );
             Inc( R1.Left );
          end;
          if beTop in fBevelEdges then
          begin
             C.Pen.Color := C1;
             C.MoveTo( R.Left, R.Top );
             C.LineTo( R.Right, R.Top );
             Inc( R1.Top );
          end;
          if beRight in fBevelEdges then
          begin
             C.Pen.Color := C2;
             C.MoveTo( R.Right - 1, R.Top );
             C.LineTo( R.Right - 1, R.Bottom );
             Dec( R1.Right );
          end;
          if beBottom in fBevelEdges then
          begin
             C.Pen.Color := C2;
             C.MoveTo( R.Left, R.Bottom - 1 );
             C.LineTo( R.Right, R.Bottom - 1 );
             Dec( R1.Bottom );
          end;
          R := R1;
       end;
    end;
begin
   OldPen := XPen.Create;
   OldPen.Assign( C.Pen );
   C.Pen.Style := psSolid;
   C.Pen.Mode := pmCopy;
   R := Rect;
   R.Right := R.Left + Width;
   R.Bottom := R.Top + Height;
   Light := ColorsMix( Color, clWhite );
   Shadow := ColorsMix( Color, clBlack );
   if fBevelOuter <> bvNone then
   begin
      C1 := Light; C2 := Shadow;
      if fBevelOuter = bvLowered then
      begin
         C1 := Shadow; C2 := Light;
      end;
      DrawBevels( C1, C2, fBevelWidth );
   end;
   if fBorderWidth <> 0 then
      DrawBevels( Color, Color, fBorderWidth );
   if fBevelInner <> bvNone then
   begin
      C1 := Light; C2 := Shadow;
      if fBevelInner = bvLowered then
      begin
         C1 := Shadow; C2 := Light;
      end;
      DrawBevels( C1, C2, fBevelWidth );
   end;
   C.Pen.Assign( OldPen ); OldPen.Free;
   OldBrush := XBrush.Create; OldBrush.Assign( C.Brush );
   C.Brush.Color := Color;
   C.Brush.Style := bsSolid;
   C.Brush.Bitmap := 0;
   C.FillRect( R );
   C.Brush.Assign( OldBrush ); OldBrush.Free;
   inherited;
end;

//////////////////////////////////////////////////////////////////
//                                                              //
//     XX    XX  XXXXXXX     XX      XXXX   XX  XXXXXX  XX      //
//      XX XX    XX    XX   XXXX     XX XX  XX  XX      XX      //
//       XX      XXXXXXX   XX  XX    XX  XX XX  XXXX    XX      //
//     XX XX     XX       XXXXXXXX   XX   XXXX  XX      XX      //
//   XX    XX    XX      XX      XX  XX    XXX  XXXXXX  XXXXXXX //
//                                                              //
//////////////////////////////////////////////////////////////////

constructor XPanel.Create( AParent : XClass );
begin
   inherited;
   BevelInner := bvNone;
   BorderWidth := 0;
   fTitle := NewLabel( nil, '', 0, 0 ); //XLabel.Create( nil );
   fTitle.OwnerForm := OwnerForm;
   fTitle.OnChangeAppearance := AppearanceChanged;
end;

destructor XPanel.Destroy;
begin
   fTitle.Free;
   inherited;
end;

function XPanel.GetCaption : String;
begin
   Result := fTitle.Caption;
end;

procedure XPanel.SetCaption( Value : String );
begin
   if Caption = Value then Exit;
   fTitle.Caption := Value;
   Invalidate;
end;

procedure XPanel.SetLayout( Value : XPanelLayout );
begin
   if fLayout = Value then Exit;
   fLayout := Value;
   Invalidate;
end;

procedure XPanel.SetColor( Value : XColor );
begin
   if Color = Value then Exit;
   inherited SetColor( Value );
   fTitle.Color := Value;
end;

function XPanel.GetFont : XFont;
begin
   Result := fTitle.Font;
end;

procedure XPanel.AppearanceChanged( Sender : TObject );
begin
   Invalidate;
end;

procedure XPanel.SetOwnerForm( Value : XForm );
begin
   inherited SetOwnerForm( Value );
   fTitle.OwnerForm := OwnerForm;
end;

procedure XPanel.PaintTo( C : XCanvas; Rect : TRect );
var R, Sz : TRect;
begin
   R := ClientRect;
   OffsetRect( R, Rect.Left, Rect.Top );
   Sz := fTitle.ClientRect;
   case fLayout of
   plCenter, plLeft, plRight :
             R.Top := R.Top + ((R.Bottom - R.Top) - Sz.Bottom) div 2;
   plBottomLeft, plBottom, plBottomRight:
             if R.Top < R.Bottom - Sz.Bottom then
                R.Top := R.Bottom - Sz.Bottom;
   end;
   if R.Bottom > R.Top + Sz.Bottom then
      R.Bottom := R.Top + Sz.Bottom;
   case fLayout of
   plCenter, plTop, plBottom:
             R.Left := R.Left + ((R.Right - R.Left) - Sz.Right) div 2;
   plRight, plTopRight, plBottomRight:
             if R.Left < R.Right - Sz.Right then
                R.Left := R.Right - Sz.Right;
   end;
   if R.Right > R.Left + Sz.Right then
      R.Right := R.Left + Sz.Right;
   fTitle.DoPaint( C.Handle, R );
   inherited;
end;

//////////////////////////////////////////////////////////////////
//                                                              //
//     XX    XX   XXXXX   XXXXXXX    XXXXX   XX   XX  XXXXXXX   //
//      XX XX    XX       XX    XX  XX   XX  XX   XX  XX    XX  //
//       XX      XX  XXX  XXXXXXX   XX   XX  XX   XX  XXXXXXX   //
//     XX XX     XX   XX  XX  XX    XX   XX  XX   XX  XX        //
//   XX    XX     XXXXX   XX   XX    XXXXX    XXXXX   XX        //
//                                                              //
//////////////////////////////////////////////////////////////////

CONSTRUCTOR XGroup.Create( AParent : XClass );
begin
   inherited;
   inherited BevelInner := bvNone;
   inherited BevelOuter := bvNone;
   inherited BevelWidth := 0;
   inherited BorderWidth := 0;
   fTitle := NewLabel( nil, '', 0, 0 ); //XLabel.Create( nil );
   fTitle.OnChangeAppearance := AppearanceChanged;
   fTitle.OwnerForm := OwnerForm;
   fFrame := XBevel.Create( nil );
   fFrame.OwnerForm := OwnerForm;
   fFrame.BevelOuter := bvLowered;
   fFrame.BevelInner := bvRaised;
   fFrame.BorderWidth := 0;
end;

destructor XGroup.Destroy;
begin
   fTitle.Free;
   fFrame.Free;
   inherited;
end;

function XGroup.GetBevel : XBevelStyle;
begin
   Result := fFrame.BevelOuter;
end;

procedure XGroup.SetBevel( Value : XBevelStyle );
begin
   if Bevel = Value then Exit;
   fFrame.BevelOuter := Value;
   case Value of
   bvLowered : fFrame.BevelInner := bvRaised;
   bvRaised  : fFrame.BevelInner := bvLowered;
   bvNone : fFrame.BevelOuter := bvNone;
   end;
   Invalidate;
end;

procedure XGroup.SetLayout( Value : XGroupLayout );
begin
   if fLayout = Value then Exit;
   fLayout := Value;
   case fLayout of
   glLeftTop, glLeftCenter, glLeftBottom :
      Font.Orientation := 900;
   glRightTop, glRightCenter, glRightBottom :
      Font.Orientation := 2700;
   else
      Font.Orientation := 0;
   end;
   Invalidate;
end;

function XGroup.GetCaption : String;
begin
   Result := fTitle.Caption;
end;

procedure XGroup.SetCaption( Value : String );
begin
   if Value = Caption then Exit;
   fTitle.Caption := Value;
   Invalidate;
end;

function XGroup.GetFont : XFont;
begin
   Result := fTitle.Font;
end;

procedure XGroup.SetColor( Value : XColor );
begin
   if Color = Value then Exit;
   inherited SetColor( Value );
   fTitle.Color := Value;
   fFrame.Color := Value;
end;

procedure XGroup.AppearanceChanged( Sender : TObject );
begin
   Invalidate;
end;

procedure XGroup.SetOwnerForm( Value : XForm );
begin
   inherited SetOwnerForm( Value );
   fTitle.OwnerForm := OwnerForm;
   fFrame.OwnerForm := OwnerForm;
end;

function XGroup.ClientRect : TRect;
var W, H : Integer;
begin
   Result := inherited ClientRect;
   Inc( Result.Left, 2 );
   Inc( Result.Top, 2 );
   Dec( Result.Right, 2 );
   Dec( Result.Bottom, 2 );
   if Caption <> '' then
   begin
      W := fTitle.Width;
      H := fTitle.Height;
      case fLayout of
         glTopLeft, glTopCenter, glTopRight :
            if H > 2 then
               Inc( Result.Top, H - 2 );
         glBottomLeft, glBottomCenter, glBottomRight:
            if H > 2 then
               Dec( Result.Bottom, H - 2 );
         glLeftTop, glLeftCenter, glLeftBottom:
            if W > 2 then
               Inc( Result.Left, W - 2 );
         glRightTop, glRightCenter, glRightBottom:
            if W > 2 then
               Dec( Result.Right, W - 2 );
      end;
   end;
   if (Result.Right < Result.Left) or (Result.Bottom < Result.Top) then
   begin
      Result.Right := Result.Left;
      Result.Bottom := Result.Top;
   end;
end;

procedure XGroup.PaintTo( C : XCanvas; Rect : TRect );
var R, LR : TRect;
    W, H : Integer;
    procedure PlaceTitle( Horz, Vert : XTextAlignment;
                          MinX, MinY, MaxX, MaxY : Integer );
    begin
       LR.Left := MinX; LR.Top := MinY;
       case Horz of
       taRight: LR.Left := MaxX - W;
       taCenter: LR.Left := MinX + (MaxX - MinX - W) div 2;
       end;
       if LR.Left < MinX then LR.Left := MinX;
       LR.Right := LR.Left + W;
       if LR.Right > MaxX then LR.Right := MaxX;

       case Vert of
       taRight: LR.Top := MaxY - H;
       taCenter: LR.Top := MinY + (MaxY - MinY - H) div 2;
       end;
       if LR.Top < MinY then LR.Top := MinY;
       LR.Bottom := LR.Top + H;
       if LR.Bottom > MaxY then LR.Bottom := MaxY;
    end;
begin
   R := Rect;
   R.Right := R.Left + Width;
   R.Bottom := R.Top + Height;
   if Caption <> '' then
   begin
      W := fTitle.Width;
      H := fTitle.Height;
      case fLayout of
         glTopLeft: PlaceTitle( taLeft, taLeft, R.Left + 6, R.Top, R.Right - 6, R.Bottom - 2 );
         glTopCenter: PlaceTitle( taCenter, taLeft, R.Left + 6, R.Top, R.Right - 6, R.Bottom - 2 );
         glTopRight: PlaceTitle( taRight, taLeft, R.Left + 6, R.Top, R.Right - 6, R.Bottom - 2 );
         glBottomLeft: PlaceTitle( taLeft, taRight, R.Left + 6, R.Top + 2, R.Right - 6, R.Bottom );
         glBottomCenter: PlaceTitle( taCenter, taRight, R.Left + 6, R.Top + 2, R.Right - 6, R.Bottom );
         glBottomRight: PlaceTitle( taRight, taRight, R.Left + 6, R.Top + 2, R.Right - 6, R.Bottom );
         glLeftBottom: PlaceTitle( taLeft, taRight, R.Left, R.Top + 6, R.Right - 2, R.Bottom - 6 );
         glLeftCenter: PlaceTitle( taLeft, taCenter, R.Left, R.Top + 6, R.Right - 2, R.Bottom - 6 );
         glLeftTop: PlaceTitle( taLeft, taRight, R.Left, R.Top + 6, R.Right - 2, R.Bottom - 6 );
         glRightTop: PlaceTitle( taRight, taLeft, R.Left + 2, R.Top + 6, R.Right, R.Bottom - 6 );
         glRightCenter: PlaceTitle( taRight, taCenter, R.Left + 2, R.Top + 6, R.Right, R.Bottom - 6 );
         glRightBottom: PlaceTitle( taRight, taRight, R.Left + 2, R.Top + 6, R.Right, R.Bottom - 6 );
      end;
      fTitle.DoPaint( C.Handle, LR );
      ExcludeUpdRect( C.Handle, LR );
      case fLayout of
         glTopLeft, glTopCenter, glTopRight:
            Inc( R.Top, (H - 2) div 2 );
         glBottomLeft, glBottomCenter, glBottomRight:
            Dec( R.Bottom, (H - 2) div 2 );
         glLeftTop, glLeftCenter, glLeftBottom:
            Inc( R.Left, (W - 2) div 2 );
         glRightTop, glRightCenter, glRightBottom:
            Dec( R.Right, (W - 2) div 2 );
      end;
   end;
   fFrame.Width := R.Right - R.Left;
   fFrame.Height := R.Bottom - R.Top;
   fFrame.DoPaint( C.Handle, R );
   inherited;
end;

//////////////////////////////////////////////////////////////////
//                                                              //
//     XX    XX  XXXXXXX     XX      XXXX  XXXX   XX  XXXXXX    //
//      XX XX    XX    XX   XXXX      XX   XX XX  XX    XX      //
//       XX      XXXXXXX   XX  XX     XX   XX  XX XX    XX      //
//     XX XX     XX       XXXXXXXX    XX   XX   XXXX    XX      //
//   XX    XX    XX      XX      XX  XXXX  XX    XXX    XX      //
//                                                              //
//////////////////////////////////////////////////////////////////

procedure XPaint.PaintTo( Canvas : XCanvas; Rect : TRect );
begin
   if Assigned( fOnPaint ) then
      fOnPaint( Self, Canvas, Rect );
   inherited;
end;

/////////////////////////////////////////////////////////////
//                                                         //
//     XX    XX   XXXXX   XX   XX  XXXXXX  XXXXX   XX  XX  //
//      XX XX    XX   XX  XX   XX  XX     XX   XX  XX XX   //
//       XX      XX       XXXXXXX  XXXXX  XX       XXXX    //
//     XX XX     XX   XX  XX   XX  XX     XX   XX  XX XX   //
//   XX    XX     XXXXX   XX   XX  XXXXXX  XXXXX   XX  XX  //
//                                                         //
/////////////////////////////////////////////////////////////

CONSTRUCTOR XCustomCheckBox.Create( AParent : XClass );
begin
   inherited;
   fPaintColor := clWindow;
   fCheckColor := clWindowText;
   fGrayDisabled := True;
   BevelInner := bvLowered;
   BevelOuter := bvNone;
   BorderWidth := 0;
   BevelWidth := 2;
   Width := 12;
   Height := 12;
end;

procedure XCustomCheckBox.Set_Color( Value : XColor );
begin
   if Color = Value then Exit;
   fPaintColor := Value;
   Invalidate;
end;

procedure XCustomCheckBox.SetCheckColor( Value : XColor );
begin
   if Value = fCheckColor then Exit;
   fCheckColor := Value;
   if Checked then Invalidate;
end;

procedure XCustomCheckBox.SetGrayDisabled( Value : Boolean );
begin
   if Value = fGrayDisabled then Exit;
   fGrayDisabled := Value;
   if not Enabled then Invalidate;
end;

procedure XCustomCheckBox.SetEnabled( Value : Boolean );
begin
   if Value = Enabled then Exit;
   inherited SetEnabled( Value );
   if not Enabled then
   if fGrayDisabled then
      Invalidate;
end;

procedure XCustomCheckBox.SetChecked( Value : Boolean );
begin
   if Value = fChecked then Exit;
   fChecked := Value;
   Invalidate;
   if Assigned( fOnChanged ) then
      fOnChanged( Self );
end;

procedure XCustomCheckBox.SetGroup( Value : XControl );
begin
   if Value = fGroup then Exit;
   fGroup := Value;
   if Checked then
      RadioAlternate( True );
end;

function XCustomCheckBox.RadioAlternate( UnCheck : Boolean ) : Boolean;
   procedure Alternate( V : XVisual );
   var I : Integer;
       C : XClass;
       R : XCustomCheckBox;
   begin
      for I := 0 to V.ChildsCount - 1 do
      begin
         C := V.Childs[ I ];
         if C is XCustomCheckBox then
         begin
            R := C as XCustomCheckBox;
            if R <> Self then
            if R.fGroup = fGroup then
            begin
               Result := True;
               if UnCheck then
                  R.Checked := False;
            end;
         end
            else
            if C is XVisual then
            Alternate( C as XVisual );
      end;
   end;
begin
   Result := False;
   if fGroup = nil then Exit;
   if ParentForm <> nil then
      Alternate( ParentForm );
end;

procedure XCustomCheckBox.DoMouseDown( Button : XMouseButton; Shift : XShiftState;
                                       X, Y : Integer );
begin
   inherited;
   if Button = mbLeft then
      Click;
end;

function XCustomCheckBox.CanFocus : Boolean;
begin
   Result := True;
end;

procedure XCheckBox.PaintTo( C : XCanvas; Rect : TRect );
var R : TRect;
    Rgn1, Rgn2 : HRgn;
    OldBrush : XBrush;
    FillColor : XColor;
begin
   R := ClientRect;
   OffsetRect( R, Rect.Left, Rect.Top );
   OldBrush := XBrush.Create;
   OldBrush.Assign( C.Brush );
   if Checked then
   begin
      Rgn1 := CreateRectRgn( 1, 3, 3, 5 );
      Rgn2 := CreateRectRgn( 2, 5, 4, 7 );
      CombineRgn( Rgn1, Rgn1, Rgn2, RGN_OR );
      DeleteObject( Rgn2 );
      Rgn2 := CreateRectRgn( 4, 4, 5, 6 );
      CombineRgn( Rgn1, Rgn1, Rgn2, RGN_OR );
      DeleteObject( Rgn2 );
      Rgn2 := CreateRectRgn( 5, 3, 6, 5 );
      CombineRgn( Rgn1, Rgn1, Rgn2, RGN_OR );
      DeleteObject( Rgn2 );
      Rgn2 := CreateRectRgn( 6, 2, 7, 4 );
      CombineRgn( Rgn1, Rgn1, Rgn2, RGN_OR );
      DeleteObject( Rgn2 );
      OffsetRgn( Rgn1, R.Left, R.Top );
      C.Brush.Color := CheckColor;
      FillRgn( C.Handle, Rgn1, C.Brush.Handle );
      ExcludeUpdRgn( C.Handle, Rgn1 );
      DeleteObject( Rgn1 );
   end;
   FillColor := BoxColor;
   if not Enabled then
   if GrayDisabled then
      FillColor := ColorsMix( FillColor, $808080 );
   C.Brush.Color := FillColor;
   C.FillRect( R );
   C.Brush.Assign( OldBrush ); OldBrush.Free;
   ExcludeUpdRect( C.Handle, R );
   inherited;
end;

constructor XRadioBox.Create( AParent : XClass );
begin
   inherited;
   BevelInner := bvNone;
   BevelWidth := 0;
   Width := 14;
   Height := 14;
   if AParent <> nil then
   if AParent is XControl then
      fGroup := AParent as XControl;
end;

procedure XRadioBox.PaintTo( C : XCanvas; Rect : TRect );
var D : Integer;
    OldPen : XPen;
    Clr : XColor;
    R : TRect;
    procedure DrawRomb( I1, I2 : Integer; Light, Shadow : XColor );
    var I : Integer;
    begin
       for I := I1 to I2 do
       begin
          C.Pen.Color := Shadow;
          C.MoveTo( R.Left + D - 2, R.Top + I - 1 );
          C.LineTo( R.Left + I - 1, R.Top + D - 2 );
          C.LineTo( R.Left + D - 2, R.Bottom - I - 1 );
          C.Pen.Color := Light;
          C.LineTo( R.Right - I - 1, R.Top + D - 2 );
          C.LineTo( R.Left + D - 2, R.Top + I - 1 );
          C.LineTo( R.Left + D - 1, R.Top + I );
       end;
    end;

begin
   R := Rect;
   R.Right := R.Left + Width;
   R.Bottom := R.Top + Height;
   OldPen := XPen.Create; OldPen.Assign( C.Pen );
   C.Pen.Style := psSolid;
   C.Pen.Mode := pmCopy;
   C.Pen.Width := 1;
   C.Pen.Color := Color;
   for D := 0 to (R.Right - R.Left) div 2 do
   begin
      C.MoveTo( R.Left, R.Top + D - 1 );
      C.LineTo( R.Left + D, R.Top - 1 );
      C.MoveTo( R.Right - 1, R.Top + D - 1 );
      C.LineTo( R.Right - 1 - D, R.Top - 1 );
      C.MoveTo( R.Left, R.Bottom - D );
      C.LineTo( R.Left + D, R.Bottom );
      C.MoveTo( R.Right - 1, R.Bottom - D - 1 );
      C.LineTo( R.Right - D - 2, R.Bottom );
   end;
   DrawRomb( 1, 2, ColorsMix( Color, clWhite ), ColorsMix( Color, clBlack ) );
   DrawRomb( 3, 4, BoxColor, BoxColor );
   Clr := BoxColor;
   if Checked then Clr := CheckColor;
   DrawRomb( 5, D - 1, Clr, Clr );
   ExcludeUpdRect( C.Handle, R );
end;

procedure XRadioBox.Click;
begin
   if Checked then
   if RadioAlternate( False ) then Exit;
   inherited;
end;

procedure XRadioBox.SetChecked( Value : Boolean );
begin
   if Value = Checked then Exit;
   inherited SetChecked( Value );
   if Checked then
      RadioAlternate( True );
end;

constructor XCustomCheck.Create( AParent : XClass );
begin
   inherited;
   BevelInner := bvNone;
   BevelWidth := 0;
   fAutoSize := True;
   fTitle := NewLabel( nil, '', 0, 0 ); //XLabel.Create( nil );
   fTitle.OwnerForm := OwnerForm;
   fTitle.OnChangeAppearance := Resize;
   CreateBox;
end;

procedure XCustomCheck.CreateBox;
begin
   if fBox = nil then Free;
   fBox.OwnerForm := OwnerForm;
   Width := fBox.Width + 6;
   Height := fBox.Height + 6;
   fLayout := blLeft;
   Layout := blRight;
end;

destructor XCustomCheck.Destroy;
begin
  fTitle.Free;
  fBox.Free;
  inherited;
end;

function XCustomCheck.GetCaption : String;
begin
   Result := fTitle.Caption;
end;

procedure XCustomCheck.SetCaption( Value : String );
begin
   fTitle.Caption := Value;
   Resize( Self );
end;

function XCustomCheck.GetFont : XFont;
begin
   Result := fTitle.Font;
end;

procedure XCustomCheck.SetAutoSize( Value : Boolean );
begin
   if Value = fAutoSize then Exit;
   fAutoSize := Value;
   Resize( Self );
end;

procedure XCustomCheck.SetLayout( Value : XBoxLayout );
var M : Integer;
begin
   //if Value = fLayout then Exit;
   fLayout := Value;
   M := 4;
   if OwnerForm <> nil then
      M := OwnerForm.Margin;
   case fLayout of
      blRight: begin
                  fTitle.Font.Orientation := 0;
                  fBox.Position := XPoint( 3, (Height - fBox.Height) div 2 );
                  fTitle.Position := XPoint( fBox.Left + fBox.Width + M,
                                             (Height - fTitle.Height) div 2 );
               end;
      blLeft:  begin
                  fTitle.Font.Orientation := 0;
                  fBox.Position := XPoint( Width - 3 - fBox.Width, (Height - fBox.Height) div 2 );
                  fTitle.Position := XPoint( 3, (Height - fTitle.Height) div 2 );
                  fTitle.Width := fBox.Left - M - fTitle.Left;
               end;
      blTop:   begin
                  fTitle.Font.Orientation := 900;
                  fBox.Position := XPoint( (Width - fBox.Width) div 2, Height - 3 - fBox.Height );
                  fTitle.Position := XPoint( (Width - fTitle.Width) div 2, 3 );
                  fTitle.Height := fBox.Top - M - fTitle.Top;
               end;
      blBottom:begin
                  fTitle.Font.Orientation := 2700;
                  fBox.Position := XPoint( (Width - fBox.Width) div 2, 3 );
                  fTitle.Position := XPoint( (Width - fTitle.Width) div 2, fBox.Top + fBox.Height + M );
               end;
   end;
   Resize( Self );
   Invalidate;
end;

procedure XCustomCheck.SetOwnerForm( Value : XForm );
begin
   inherited;
   fTitle.OwnerForm := OwnerForm;
   fBox.OwnerForm := OwnerForm;
end;

procedure XCustomCheck.SetChecked( Value : Boolean );
begin
   if Checked = Value then Exit;
   fBox.Checked := Value;
   inherited SetChecked( Value );
end;

procedure XCustomCheck.Resize( Sender : TObject );
var M : Integer;
    F : XForm;
    R : TRect;
    W, H : Integer;
begin
   if not fAutoSize then Exit;
   M := 4;
   F := OwnerForm;
   if F <> nil then M := F.Margin;
   R := BoundsRect;
   W := R.Right - R.Left;
   H := R.Bottom - R.Top;
   case fLayout of
   blLeft, blRight:
      begin
         W := fTitle.Width + M + fBox.Width + 6;
         H := fTitle.Height + 6;
         if H < fBox.Height + 4 then
            H := fBox.Height + 4;
      end;
   blTop, blBottom:
      begin
         H := fTitle.Height + M + fBox.Height + 6;
         W := fTitle.Width + 6;
         if W < fBox.Width + 4 then
            W := fBox.Width + 4;
      end;
   end;
   if (W <> Width) or (H <> Height) then
   begin
      R.Right := R.Left + W;
      R.Bottom := R.Top + H;
      BoundsRect := R;
      SetLayout( fLayout );
      Invalidate;
   end;
end;

procedure XCustomCheck.PaintTo( C : XCanvas; Rect : TRect );
var R, TR, BR : TRect;
begin
   R := Rect;
   R.Right := R.Left + Width;
   R.Bottom := R.Top + Height;
   BR := fBox.BoundsRect;
   OffsetRect( BR, R.Left, R.Top );
   TR := fTitle.BoundsRect;
   OffsetRect( TR, R.Left, R.Top );
   fBox.DoPaint( C.Handle, BR );
   fTitle.DoPaint( C.Handle, TR );
   if Focused then
   begin
      Inc( R.Top ); Inc( R.Left );
      Dec( R.Bottom, 2 ); Dec( R.Right, 2 );
      if GlobalFormsManager <> nil then
      if Assigned( GlobalFormsManager.OnDrawFocusRect ) then
         GlobalFormsManager.OnDrawFocusRect( Self, R, Color );
   end;
   inherited;
end;

procedure XCheck.CreateBox;
begin
   fBox := XCheckBox.Create( nil );
   inherited;
end;

constructor XRadio.Create( AParent : XClass );
begin
   inherited;
   if AParent <> nil then
   if AParent is XControl then
      SetGroup( AParent as XControl );
end;

procedure XRadio.CreateBox;
begin
   fBox := XRadioBox.Create( nil );
   inherited;
end;

procedure XRadio.Click;
begin
   if Checked then
   if RadioAlternate( False ) then Exit;
   inherited;
end;

procedure XRadio.SetChecked( Value : Boolean );
begin
   if Value = Checked then Exit;
   inherited SetChecked( Value );
   if Checked then
      RadioAlternate( True );
end;

{ XCustomButton }

function XCustomButton.CanFocus: Boolean;
begin
   Result := not FSpeedButton;
end;

constructor XCustomButton.Create(AParent: XClass);
begin
   inherited;
   BevelInner := bvNone;
   BorderWidth := 0;
   Height := DefFontHeight + 6;
   fRepeats := True;
end;

procedure XCustomButton.DoKeyDown(var Key: Word; Shift: XShiftState);
begin
   if (Key = VK_RETURN) or (Key = VK_SPACE) then
   begin
      fKeyUsed := Key;
      if Fixed then
         Down := not Down
      else
      begin
         if not Down then
            Down := True
         else
         if FRepeats then
         if Assigned( fOnClick ) then
            fOnClick( Self );
      end;
      Key := 0;
   end;
end;

procedure XCustomButton.DoKeyUp(var Key: Word; Shift: XShiftState);
begin
   if Key = fKeyUsed then
   begin
      if not Fixed then
         Down := False;
      Key := 0;
   end;
end;

procedure XCustomButton.DoMouseDown(Button: XMouseButton;
  Shift: XShiftState; X, Y: Integer);
begin
  if Fixed then
     Down := not Down
  else
  begin
     Down := True;
     SetCapture( ParentForm.Handle );
  end;
  inherited;
end;

procedure XCustomButton.DoMouseUp(Button: XMouseButton; Shift: XShiftState;
  X, Y: Integer);
begin
   if not Fixed then
      Down := False;
end;

procedure XCustomButton.SetDown(const Value: Boolean);
begin
  if FDown = Value then Exit;
  FDown := Value;
  if Fixed or Down then
  if Assigned( fOnClick ) then
     fOnClick( Self );
  if FDown then
  begin
     BorderWidth := 0;
     if not FFlat then
        BevelOuter := bvLowered;
  end
     else
  begin
     if FFlat then
     begin
        {
        BevelOuter := bvNone;
        BorderWidth := BevelWidth;
        }
     end
        else
     begin
        BorderWidth := 0;
        BevelOuter := bvRaised;
     end;
  end;
end;

procedure XCustomButton.SetFixed(const Value: Boolean);
begin
  //if FFixed = Value then Exit;
  FFixed := Value;
end;

procedure XCustomButton.SetFlat(const Value: Boolean);
begin
   if fFlat = Value then Exit;
   fFlat := Value;
   if fFlat then
   begin
      //if not fDown then
      begin
         BevelOuter := bvNone;
         BorderWidth := 0;
      end;
   end
      else
   begin
      BorderWidth := BevelWidth;
      if fDown then
         BevelOuter := bvLowered
      else
         BevelOuter := bvRaised;
   end;
end;

procedure XCustomButton.SetSpeedButton(const Value: Boolean);
begin
  FSpeedButton := Value;
  Focused := False;
end;

{ XButton }

constructor XButton.Create(AParent: XClass);
begin
   inherited;
   fTitle := NewLabel( nil, '', 0, 0 ); //XLabel.Create( nil );
   fTitle.OwnerForm := OwnerForm;
   fTitle.OnChangeAppearance := Resize;
   fAutoSize := True;
end;

destructor XButton.Destroy;
begin
   fTitle.Free;
   inherited;
end;

function XButton.GetCaption: String;
begin
   Result := fTitle.Caption;
end;

function XButton.GetFont: XFont;
begin
   Result := fTitle.Font;
end;

procedure XButton.PaintTo(C: XCanvas; Rect: TRect);
var R : TRect;
begin
   R := fTitle.BoundsRect;
   OffsetRect( R, Rect.Left + 3, Rect.Top + 3 );
   if R.Right > Rect.Right - 1 then
      R.Right := Rect.Right - 1;
   if R.Bottom > Rect.Bottom - 1 then
      R.Bottom := Rect.Bottom - 1;
   fTitle.PaintTo( C, R );
   if Focused then
   begin
      Dec( R.Left, 1 ); Dec( R.Top, 1 );
      Inc( R.Right, 1 ); Inc( R.Bottom, 1 );
      if GlobalFormsManager <> nil then
      if Assigned( GlobalFormsManager.OnDrawFocusRect ) then
         GlobalFormsManager.OnDrawFocusRect( Self, R, Color );
      {
      OldBrush := XBrush.Create;
      OldBrush.Assign( C.Brush );
      C.Brush.Color := Color;
      C.Brush.Style := bsSolid;
      C.Brush.Bitmap := 0;
      Inc( R.Top ); Inc( R.Left );
      Dec( R.Bottom, 2 ); Dec( R.Right, 2 );
      C.FillRect( R );
      OldRgn := CreateRectRgn( 0, 0, 0, 0 );
      GetClipRgn( C.Handle, OldRgn );
      ExtSelectClipRgn( C.Handle, 0, RGN_COPY );
      C.DrawFocusRect( R );
      ExtSelectClipRgn( C.Handle, OldRgn, Rgn_COPY );
      DeleteObject( OldRgn );
      C.Brush.Assign( OldBrush );
      OldBrush.Free;
      ExcludeUpdRect( C.Handle, R );
      }
   end;
   inherited;
end;

procedure XButton.Resize( Sender : TObject );
begin
   if not fAutoSize then Exit;
   if Caption = '' then
   begin
      Width := 64;
      Height := DefFontHeight + 6;
      Exit;
   end;
   Width := fTitle.Width + 6;
   Height := fTitle.Height + 6;
end;

procedure XButton.SetAutoSize(const Value: Boolean);
begin
   if fAutoSize = Value then Exit;
   fAutoSize := Value;
   if Value then Resize( Self );
end;

procedure XButton.SetCaption(const Value: String);
begin
   if Caption = Value then Exit;
   fTitle.Caption := Value;
   Invalidate;
   if fAutoSize then Resize( Self );
end;

procedure XCustomCheckBox.DoKeyDown(var Key: Word; Shift: XShiftState);
begin
   if Key = VK_RETURN then
   begin
      Click;
      Key := 0;
   end
      else
   if Key = VK_SPACE then
   begin
      Click;
      if ParentForm <> nil then
         ParentForm.GotoControl( VK_Tab );
      Key := 0;
   end;
end;

procedure XCustomCheckBox.Click;
begin
   Checked := not Checked;
end;

procedure XButton.SetColor(Value: XColor);
begin
   if Value = Color then Exit;
   fTitle.Color := Value;
   inherited;
end;

function NewLabel( AParent : XClass; Caption : String; X, Y : Integer ) : XLabel;
begin
  Result := nil;
  if Assigned( GlobalControlsManager.OnCreateLabel ) then
     GlobalControlsManager.OnCreateLabel( AParent, Caption, X, Y, Result );
  if Result = nil then
  begin
     Result := XLabel.Create( AParent );
     Result.Position := XPoint( X, Y );
     Result.Caption := Caption;
  end;
end;

initialization
  GlobalControlsManager := XControlsManager.Create;

finalization
  GlobalControlsManager.Free;

end.
