unit XFocusDrawing;

interface

uses Windows, XClasses, XVisuals, XForms, XControls, XGraphics, XCanvases;

type
   XFocusDraw = class
   private
   protected
      constructor Create;
      procedure DrawControlFocusRect( Sender : TObject; Rect : TRect; Color : XColor );
   public
   end;

var FocusPainter : XFocusDraw;

procedure UseFocusPainter;
// Call UseFocusPainter in YourProject.dpr before creating of
// first form to provide drawing of focus rectangles around
// focused buttons and checkboxes. If this module is not used,
// applet will be smaller on about 1,5 Kbytes.

// If You want provide drawing of focus rect for Your own
// self-painting controls (i.g. derived from XCustomControl and
// from its descenders), use code similar following in PaintTo
// implementation:
//
//  if GlobalFormsManager <> nil then
//  if Assigned( GlobalFormsManager.OnDrawFocusRect ) then
//     GlobalFormaManager.OnDrawFocusRect( Self, Rect, Color );
//
// This will clear all non-clipped area overlapped by Rect, so
// make this call AFTER painting all internal elements if any.
// After drawing the focus given Rect area becomes clipped (e.g.
// it is excluded from area available for consequences painting
// operations).

implementation

procedure UseFocusPainter;
begin
   if FocusPainter <> nil then Exit;
   FocusPainter := XFocusDraw.Create;
end;

constructor XFocusDraw.Create;
begin
   inherited;
   if GlobalFormsManager = nil then
      GlobalFormsManager := XFormsManager.Create;
   GlobalFormsManager.OnDrawFocusRect := DrawControlFocusRect;
end;

procedure XFocusDraw.DrawControlFocusRect( Sender : TObject; Rect : TRect; Color : XColor );
var F : XForm;
    C : XCanvas;
    OldBrush : XBrush;
    OldRgn : HRgn;
begin
   F := (Sender as XControl).OwnerForm;
   if F = nil then Exit;
   C := WindowCanvas( F );
      OldBrush := XBrush.Create;
      OldBrush.Assign( C.Brush );
      C.Brush.Color := Color;
      C.Brush.Style := bsSolid;
      C.Brush.Bitmap := 0;
      C.FillRect( Rect );
      OldRgn := CreateRectRgn( 0, 0, 0, 0 );
      GetClipRgn( C.Handle, OldRgn );
      ExtSelectClipRgn( C.Handle, 0, RGN_COPY );
      C.DrawFocusRect( Rect );
      ExtSelectClipRgn( C.Handle, OldRgn, Rgn_COPY );
      DeleteObject( OldRgn );
      C.Brush.Assign( OldBrush );
      OldBrush.Free;
      (Sender as XControl).ExcludeUpdRect( C.Handle, Rect );
end;

end.
