unit XMenus;

interface

uses Windows, Messages, XClasses, XVisuals, XWindows, XForms;

type
   XOnMenuItem = procedure( Sender : TObject; Item : Integer ) of object;

   XMenu = class( XChild )
   private
    FHandle: HMenu;
    FItemHandles : ZList;
    FFirstCommand : Integer;
    FOldOnMessage : XOnFormMessage;
    FOnMenuItem : XOnMenuItem;
    FOnRadioOff : XOnMenuItem;
     procedure GetItemState( Item : Integer; var MII : TMenuItemInfo );
     function GetItemChecked( Item : Integer ) : Boolean;
     procedure SetItemChecked( Item : Integer; Value : Boolean );
   protected
     function FillMenuItems( FHandle : HMenu; StartIdx : Integer; Template : array of String )
              : Integer;
     procedure CreateMenuItems( Template : array of String ); virtual;
     procedure DispatchMessage( Sender : TObject; var Msg : TMsg;
                                var Rslt : Integer; var Handled : Boolean );
     procedure Initialize; virtual;
   public
     constructor Create( AParent : XClass; FirstCmd : Integer; Template : array of String ); reintroduce;
     destructor Destroy; override;
     property Handle : HMenu read FHandle;
     function GetMenuItemHandle( Idx : Integer ) : HMenu;
     //function GetMenuItemPosition( Idx : Integer ) : Integer;
     property FirstCommand : Integer read FFirstCommand;
     property OnMenuItem : XOnMenuItem read FOnMenuItem write FOnMenuItem;
     property OnUncheckRadioItem : XOnMenuItem read FOnRadioOff write FOnRadioOff;
     property ItemChecked[ Idx : Integer ] : Boolean read GetItemChecked
              write SetItemChecked;
     procedure RadioCheck( Idx : Integer );
   end;

   XPopup = class( XMenu )
   private
   protected
     procedure Initialize; override;
     procedure CreateMenuItems( Template : array of String ); override;
   public
     procedure Popup( X, Y : Integer );
   end;

implementation

{ XMenu }

constructor XMenu.Create(AParent: XClass; FirstCmd : Integer; Template: array of String);
begin
   inherited Create( AParent );
   FFirstCommand := FirstCmd;
   FItemHandles := ZList.Create;
   CreateMenuItems( Template );
   Initialize;
end;

destructor XMenu.Destroy;
var F : XForm;
begin
   F := GetClassParentForm( Self );
   if (F = nil) or (F.Menu <> FHandle) then
      DestroyMenu( FHandle );
   FItemHandles.Free;
   inherited;
end;

procedure XMenu.Initialize;
var F : XForm;
begin
   F := GetClassParentForm( Self );
   if F <> nil then
   begin
      F.Menu := FHandle;
      FOldOnMessage := F.On_Message;
      F.On_Message := DispatchMessage;
   end;
end;

procedure XMenu.CreateMenuItems(Template: array of String);
begin
   FHandle := CreateMenu;
   FillMenuItems( FHandle, Low( Template ), Template );
end;

procedure XMenu.DispatchMessage(Sender: TObject; var Msg: TMsg;
  var Rslt: Integer; var Handled: Boolean);
begin
  if Msg.message = WM_COMMAND then
  begin
     if (Msg.lParam = 0) and (HIWORD( Msg.wParam ) = 0) then
     if (Msg.wParam >= FFirstCommand) and (Msg.wParam < FFirstCommand + FItemHandles.Count) then
     begin
        Handled := True;
        Rslt := 0;
        if Assigned( FOnMenuItem ) then
           FOnMenuItem( Self, Msg.wParam - FFirstCommand );
     end;
  end;
  if not Handled then
  if Assigned( FOldOnMessage ) then
     FOldOnMessage( Sender, Msg, Rslt, Handled );
end;

function XMenu.FillMenuItems(FHandle : HMenu; StartIdx : Integer; Template: array of String)
         : Integer;
var I : Integer;
    //N : Integer;
    S : String;
    MII : TMenuItemInfo;
    SubMenu : HMenu;
begin
   I := StartIdx;
   //N := 0;
   while I <= High( Template ) do
   begin
      //J := I;
      S := Template[ I ];
      if S = '' then break;
      if S = ')' then
      begin
         Result := I + 1;
         Exit;
      end;
      FillChar( MII, Sizeof( MII ), 0 );
      MII.cbSize := Sizeof( MII );
      MII.fMask := MIIM_DATA or MIIM_ID or MIIM_STATE or MIIM_SUBMENU or MIIM_TYPE;
      MII.fType := MFT_STRING;
      MII.fState := MFS_ENABLED;
      if S <> '-' then
      if (S[ 1 ] = '-') or (S[ 1 ] = '+') then
      begin
         if S[ 1 ] = '-' then
            MII.fState := MII.fState or MFS_UNCHECKED
         else
            MII.fState := MII.fState or MFS_CHECKED;
         S := Copy( S, 2, Length( S ) - 1 );
         if S <> '' then
         if S[ 1 ] = '!' then
         begin
            MII.fType := MII.fType or MFT_RADIOCHECK;
            S := Copy( S, 2, Length( S ) - 1 );
         end;
      end;
      MII.wID := FFirstCommand + FItemHandles.Count;
      SubMenu := 0;
      if I < High( Template ) then
      if Template[ I + 1 ] = '(' then
         SubMenu := CreatePopupMenu;
      MII.hSubMenu := SubMenu;
      if S = '-' then
      begin
         MII.fType := MFT_SEPARATOR;
         MII.fState := MFS_GRAYED;
         MII.wID := 0;
      end;
      MII.dwTypeData := PChar( S );
      MII.cch := Length( S );
      InsertMenuItem( FHandle, Cardinal(-1) {N}, True, MII );
      if S <> '-' then
         FItemHandles.Add( Pointer( FHandle ) );
      //Inc( N );
      if SubMenu <> 0 then
         I := FillMenuItems( SubMenu, I + 2, Template )
      else
         Inc( I );
   end;
   Result := I;
end;

function XMenu.GetMenuItemHandle(Idx: Integer): HMenu;
begin
   Result := HMenu( FItemHandles.Items[ Idx ] );
end;

{
function XMenu.GetMenuItemPosition(Idx: Integer): Integer;
var I : Integer;
    H : HMenu;
begin
   Result := -1;
   H := GetMenuItemHandle ( Idx );
   for I := 0 to GetMenuItemCount( H ) - 1 do
       if Integer( GetMenuItemId( H, I ) ) = Idx + FFirstCommand then
       begin
          Result := I;
          break;
       end;
end;
}

function XMenu.GetItemChecked(Item: Integer): Boolean;
var MII : TMenuItemInfo;
begin
  GetItemState( Item, MII );
  Result := (MII.fState and MFS_CHECKED) <> 0;
end;

procedure XMenu.GetItemState(Item: Integer; var MII: TMenuItemInfo);
var H : HMenu;
    //P : Integer;
begin
   FillChar( MII, Sizeof( MII ), 0 );
   MII.cbSize := Sizeof( MII );
   MII.fMask := MIIM_STATE;
   H := GetMenuItemHandle( Item );
   //P := GetMenuItemPosition( Item );
   //MII.fState := GetMenuState( H, P, MF_BYPOSITION );
   MII.fState := GetMenuState( H, Item + FFirstCommand, MF_BYCOMMAND );
end;

procedure XMenu.SetItemChecked(Item: Integer; Value: Boolean);
var MII : TMenuItemInfo;
begin
   GetItemState( Item, MII );
   MII.fState := MII.fState and not( MFS_CHECKED or MFS_UNCHECKED );
   MII.fMask := MIIM_STATE;
   if Value then
      MII.fState := MII.fState or MFS_CHECKED
   else
      MII.fState := MII.fState or MFS_UNCHECKED;
   SetMenuItemInfo( GetMenuItemHandle( Item ), FFirstCommand + Item,
                    False, MII );
end;

procedure XMenu.RadioCheck(Idx: Integer);
var H : HMenu;
    I, J, First, Last, K : Integer;
    St : Integer;
begin
   H := GetMenuItemHandle( Idx );
   First := 0;
   Last := GetMenuItemCount( H ) - 1;
   J := -1;
   for I := 0 to Last do
   begin
      St := GetMenuState( H, I, MF_BYPOSITION );
      if (St and MF_SEPARATOR) <> 0 then
      begin
         if J < 0 then
            First := I + 1
         else
         begin
            Last := I - 1;
            break;
         end;
      end;
      if Integer( GetMenuItemId( H, I ) ) = Idx + FFirstCommand then
         J := I;
   end;
   if J >= 0 then
   //if (GetMenuState( H, J, MF_BYPOSITION ) and MFS_CHECKED) = 0 then
   begin
      for I := First to Last do
      if I <> J then
      begin
         K := GetMenuItemId( H, I );
         if (GetMenuState( H, K, MF_BYCOMMAND ) and MFS_CHECKED) <> 0 then
         begin
            ItemChecked[ K - FFirstCommand ] := False;
            if Assigned( FOnRadioOff ) then
               FOnRadioOff( Self, K - FFirstCommand );
            //break;
         end;
      end;
      ItemChecked[ Idx ] := True;
   end;
end;

{ XPopup }

procedure XPopup.CreateMenuItems(Template: array of String);
begin
   FHandle := CreatePopupMenu;
   FillMenuItems( FHandle, Low( Template ), Template );
end;

procedure XPopup.Initialize;
var F : XForm;
begin
   F := GetClassParentForm( Self );
   if F <> nil then
   begin
      //F.Menu := FHandle;
      FOldOnMessage := F.On_Message;
      F.On_Message := DispatchMessage;
   end;
end;

procedure XPopup.Popup(X, Y: Integer);
var F : XForm;
begin
   F := GetClassParentForm( Self );
   if F <> nil then
   if F.Handle <> 0 then
   TrackPopupMenu( FHandle, TPM_LEFTALIGN or TPM_LEFTBUTTON,
                   X, Y, 0, F.Handle, nil );
end;

end.
