unit XRotateFonts;

interface

uses Windows, XClasses, XCanvases, XGraphics, XControls;

type
   XFontRotator = class
   private
     fOldOnPaintLabel : XOnPaintLabel;
     fPaintingLabel : Boolean;
      procedure TextArea( Sender: TObject; var Sz : TSize; var Pt : TPoint );
      procedure PaintLabel( Sender : TObject; Canvas : XCanvas;
                            Rect : TRect; var Handled : Boolean );
   protected
   public
      constructor Create;
   end;

var FontRotator : XFontRotator;

procedure UseFontRotator;

implementation

procedure UseFontRotator;
begin
   if FontRotator <> nil then Exit;
   FontRotator := XFontRotator.Create;
end;

constructor XFontRotator.Create;
begin
   if GlobalCanvasManager = nil then
      GlobalCanvasManager := XCanvasManager.Create;
   GlobalCanvasManager.OnTextArea := TextArea;
   FOldOnPaintLabel := GlobalControlsManager.On_PaintLabel;
   GlobalControlsManager.On_PaintLabel := PaintLabel;
end;

procedure XFontRotator.PaintLabel(Sender: TObject; Canvas: XCanvas;
  Rect: TRect; var Handled : Boolean);
var L : XLabel;
    DC, MemDC : HDC;
    MemBmp, OldBmp : HBitmap;
    TempCanvas : XCanvas;
    R : TRect;
begin
   L := Sender as XLabel;
   if L.Font.Orientation <> 0 then
   begin
   // Here we are painting label through buffer, avoiding
   // flickering of rotated text:
     if not fPaintingLabel then
     begin
       R := XRect( 0, 0, Rect.Right - Rect.Left, Rect.Bottom - Rect.Top );
       DC := GetDC( 0 );
       MemBmp := CreateCompatibleBitmap( DC, R.Right, R.Bottom );
       ReleaseDC( 0, DC );
       MemDC := CreateCompatibleDC( 0 );
       OldBmp := SelectObject( MemDC, MemBmp );
       TempCanvas := XCanvas.Create;
       TempCanvas.Handle := MemDC;
       fPaintingLabel := True;
       L.PaintTo( TempCanvas, R );
       fPaintingLabel := False;
       BitBlt( Canvas.Handle, Rect.Left, Rect.Top, R.Right,
               R.Bottom, TempCanvas.Handle, 0, 0, SrcCopy );
       TempCanvas.Handle := MemDC;
       TempCanvas.Free;
       SelectObject( MemDC, OldBmp );
       DeleteObject( MemBmp );
       DeleteDC( MemDC );
       Handled := True;
     end;
   end;
   if not Handled then
   if Assigned( FOldOnPaintLabel ) then
      FOldOnPaintLabel( Sender, Canvas, Rect, Handled );
end;

procedure XFontRotator.TextArea( Sender : TObject; var Sz : TSize; var Pt : TPoint );
var Orient : Integer;
    Pts : array[ 1..4 ] of TPoint;
    MinX, MinY, I : Integer;
    A : Double;
begin
   if not (Sender is XCanvas) then Exit;
   Orient := (Sender as XCanvas).Font.Orientation;
   Pt.x := 0; Pt.y := 0;
   if Orient = 0 then Exit;
   A := Orient / 1800.0 * PI;
   Pts[ 1 ] := Pt;
   Pts[ 2 ].x := Round( Sz.cx * cos( A ) );
   Pts[ 2 ].y := - Round( Sz.cx * sin( A ) );
   Pts[ 4 ].x := - Round( Sz.cy * cos( A + PI / 2 ) );
   Pts[ 4 ].y := Round( Sz.cy * sin( A + PI / 2 ) );
   Pts[ 3 ].x := Pts[ 2 ].x + Pts[ 4 ].x;
   Pts[ 3 ].y := Pts[ 2 ].y + Pts[ 4 ].y;
   MinX := 0; MinY := 0;
   for I := 2 to 4 do
   begin
      if Pts[ I ].x < MinX then
         MinX := Pts[ I ].x;
      if Pts[ I ].y < MinY then
         MinY := Pts[ I ].y;
   end;
   Sz.cx := 0;
   Sz.cy := 0;
   for I := 1 to 4 do
   begin
      Pts[ I ].x := Pts[ I ].x - MinX;
      Pts[ I ].y := Pts[ I ].y - MinY;
      if Pts[ I ].x > Sz.cx then
         Sz.cx := Pts[ I ].x;
      if Pts[ I ].y > Sz.cy then
         Sz.cy := Pts[ I ].y;
   end;
   Pt := Pts[ 1 ];
end;

end.
